
import java.util.*;

public class Bloque extends AttachableElement {

    private static int id = 0;  // Bloque identification number
    private double crossSection;
    private final float mass;
    private Vector2D pos_t;     // current position at time t
    private Vector2D pos_tPlusDelta;  // next position in delta time in future
    private Vector2D speed_t;   // speed at time t
    private Vector2D speed_tPlusDelta;   // speed in delta time in future
    private Vector2D a_t;    // acceleration at time t
    private Vector2D a_tMinusDelta;  // acceleration delta time ago;
    private Vector<Pilar> pils;  // Vector can grow, arrays cannot. 
/** 
 * Constructor Bloque.  Inicializa un nuevo Bloque
 
 */  
    private Bloque() {   // nobody can create a block without state
        super(id++);
        mass = 10;
    }
/** 
 * Constructor Bloque.  Inicializa un nuevo Bloque
 * @param mass 	Massa del Bloque
 * @param position Posicion inicial del objeto creado
 * @param speed Velocidad actual del bloque
 */  
    public Bloque(float mass, Vector2D position, Vector2D speed) {
        super(id++);
        this.mass = mass;
        pos_t = position;
        speed_t = speed;
        a_t = new Vector2D();
        a_tMinusDelta = new Vector2D();
        pils = new Vector<Pilar>();



        crossSection = 1.0f;
    }
/**
 *  Cambia la velocidad del objeto bloque
 * @param speed_t Nueva velocidad del bloque.
 */
    public void setSpeed(Vector2D speed_t) {//Modulo para setear la velocidad, se ocupa en el metodo start de MyWorld
        this.speed_t = speed_t;
    }
/**
 *  Cambia la aceleracion del objeto bloque
 * @param speed_t vector de aceleración
 */
    public void setAccel(Vector2D a_t) {//Idem a setSpeed, con la aceleracion.
        this.a_t = a_t;
    }
/**
 *  Agrega un pilar al vector de Pilar
 * @param pil Objeto pilar
 */
    public void attachPilar(Pilar pil) {
        pils.add(pil);
    }
/**
 *  Remueve un pilar al vector de Pilar
 * @param pil Objeto pilar
 */
    public void detachPilar(Pilar pil) {
        pils.remove(pil);
    }
/**
 *  Entrega el vector posicion del objeto Bloque
 * 
 */
    public Vector2D getPosition() {
        return pos_t;
    }

    public double getX() {
        return pos_t.getX();
    }

    public double getY() {
        return pos_t.getY();
    }

    public void setPosition(double x, double y) {
        pos_t.set(x, y);
    }

    public void setCrossSection(double cs) {
        crossSection = cs;
    }
/**
 *  Calcula la fuerza que actúa sobre el bloque para realizar cálculos de velocidad y posicion
 * @param g vector de aceleracion
 * @param viscosity viscosidad del medio
 */
    private Vector2D getNetForce(Vector2D g, float viscosity) {
        Vector2D force = new Vector2D();
        Vector2D force2 = new Vector2D();
        for (Pilar s : pils) {
            force = force.plus(s.getForce(this));
            force = force.plus(g.times(mass));
            force = force.minus(speed_t.times(viscosity * crossSection));
        }
        return (force);
    }
/**
 *  Calcula la aceleracion inical del bloque
 * @param g vector de aceleracion
 * @param viscosity viscosidad del medio
 */
    public void setInitialState(Vector2D g, float viscosity) {
        a_tMinusDelta = getNetForce(g, viscosity).times(1 / mass);
    }
/**
 *  Calcula la la dinámica del bloque de acuerdo al tiempo de muestre, gravedad visocidad del sistema, velocidad actual del bloque y posicion.
 * @param g vector de aceleracion
 * @param viscosity viscosidad del medio
 * @param delta_t Tiempo de muestreo entre un estado y el siguiente 
 */
    public void computeNextState(double delta_t, Vector2D g, float viscosity) {
        a_t = getNetForce(g, viscosity).times(1 / mass);
        speed_tPlusDelta = speed_t.plus(a_t.times(3).minus(a_tMinusDelta).times(delta_t / 2));
        //
        speed_tPlusDelta = speed_tPlusDelta.mult(1, 0);
        //
        pos_tPlusDelta = pos_t.plus(speed_t.times(delta_t)).plus(a_t.times(4).minus(a_tMinusDelta).times(delta_t * delta_t / 6));
    }
/**
 *  Actualiza el estado del bloque
 */
    public void updateState() {
        pos_t = pos_tPlusDelta;
        speed_t = speed_tPlusDelta;
        a_tMinusDelta = a_t;
    }
/**
 * Returna Velocidad actual del bloque
 */
    public Vector2D getSpeed() {
        return speed_t;

    }
}
